@@ -4,12 +4,6 @@ dicom2nifti
 
 @author: abrys
 """
-
-from __future__ import print_function
-import dicom2nifti.patch_pydicom_encodings
-
-dicom2nifti.patch_pydicom_encodings.apply()
-
 import os
 import re
 import traceback
@@ -53,6 +47,19 @@ def dicom_to_nifti(dicom_input, output_file=None):
 
     assert common.is_siemens(dicom_input)
 
+    # remove duplicate slices based on position and data
+    dicom_input = convert_generic.remove_duplicate_slices(dicom_input)
+
+    # remove localizers based on image type
+    dicom_input = convert_generic.remove_localizers_by_imagetype(dicom_input)
+
+    # remove_localizers based on image orientation (only valid if slicecount is validated)
+    dicom_input = convert_generic.remove_localizers_by_orientation(dicom_input)
+
+    # if no dicoms remain raise exception
+    if not dicom_input:
+        raise ConversionValidationError('TOO_FEW_SLICES/LOCALIZER')
+
     if _is_4d(dicom_input):
         logger.info('Found sequence type: MOSAIC 4D')
         return _mosaic_4d_to_nifti(dicom_input, output_file)
@@ -150,7 +157,7 @@ def _mosaic_4d_to_nifti(dicom_input, output_file):
     affine = _create_affine_siemens_mosaic(dicom_input)
     logger.info('Creating nifti')
     # Convert to nifti
-    nii_image = nibabel.Nifti1Image(full_block, affine)
+    nii_image = nibabel.Nifti1Image(full_block.squeeze(), affine)
     common.set_tr_te(nii_image, float(sorted_mosaics[0].RepetitionTime), float(sorted_mosaics[0].EchoTime))
     logger.info('Saving nifti to disk')
     # Save to disk
@@ -202,7 +209,7 @@ def _classic_4d_to_nifti(grouped_dicoms, output_file):
 
     logger.info('Creating nifti')
     # Convert to nifti
-    nii_image = nibabel.Nifti1Image(full_block, affine)
+    nii_image = nibabel.Nifti1Image(full_block.squeeze(), affine)
     common.set_tr_te(nii_image, float(grouped_dicoms[0][0].RepetitionTime), float(grouped_dicoms[0][0].EchoTime))
     logger.info('Saving nifti to disk')
     # Save to disk
@@ -235,37 +242,77 @@ def _classic_4d_to_nifti(grouped_dicoms, output_file):
             'NII': nii_image,
             'MAX_SLICE_INCREMENT': slice_increment}
 
-
 def _classic_get_grouped_dicoms(dicom_input):
     """
     Search all dicoms in the dicom directory, sort and validate them
 
     fast_read = True will only read the headers not the data
     """
-    # Loop overall files and build dict
+
     # Order all dicom files by InstanceNumber
-    if [d for d in dicom_input if 'InstanceNumber' in d]:
-        dicoms = sorted(dicom_input, key=lambda x: x.InstanceNumber)
-    else:
-        dicoms = common.sort_dicoms(dicom_input)
+    dicoms = sorted(dicom_input, key=lambda x: x.InstanceNumber)
 
     # now group per stack
-    grouped_dicoms = []
+    grouped_dicoms = [[]]  # list with first element a list
+    stack_index = 0
 
-    # loop over all sorted dicoms
-    stack_position_tag = Tag(0x0020, 0x0012)  # in this case it is the acquisition number
-    for index in range(0, len(dicoms)):
-        dicom_ = dicoms[index]
-        if stack_position_tag not in dicom_:
-            stack_index = 0
+    # loop over all sorted dicoms and sort them by stack
+    # for this we use the position and direction of the slices so we can detect a new stack easily
+    previous_position = None
+    previous_direction = None
+    for dicom_ in dicoms:
+        current_direction = None
+        # if the stack number decreases we moved to the next stack
+        if previous_position is not None:
+            current_direction = numpy.array(dicom_.ImagePositionPatient) - previous_position
+            current_direction = current_direction / numpy.linalg.norm(current_direction)
+        if current_direction is not None and \
+                previous_direction is not None and \
+                not numpy.allclose(current_direction, previous_direction, rtol=0.05, atol=0.05):
+            previous_position = numpy.array(dicom_.ImagePositionPatient)
+            previous_direction = None
+            stack_index += 1
         else:
-            stack_index = dicom_[stack_position_tag].value - 1
-        while len(grouped_dicoms) <= stack_index:
+            previous_position = numpy.array(dicom_.ImagePositionPatient)
+            previous_direction = current_direction
+
+        if stack_index >= len(grouped_dicoms):
             grouped_dicoms.append([])
         grouped_dicoms[stack_index].append(dicom_)
 
     return grouped_dicoms
 
+# old function that was replaced by the new one for icometrix/dicom2nifti#70 will keep it for now
+# def _classic_get_grouped_dicoms(dicom_input):
+#     """
+#     Search all dicoms in the dicom directory, sort and validate them
+#
+#     fast_read = True will only read the headers not the data
+#     """
+#     # Loop overall files and build dict
+#     # Order all dicom files by InstanceNumber
+#     if [d for d in dicom_input if 'InstanceNumber' in d]:
+#         dicoms = sorted(dicom_input, key=lambda x: x.InstanceNumber)
+#     else:
+#         dicoms = common.sort_dicoms(dicom_input)
+#
+#     # now group per stack
+#     grouped_dicoms = []
+#
+#     # loop over all sorted dicoms
+#     stack_position_tag = Tag(0x0020, 0x0012)  # in this case it is the acquisition number
+#     for index in range(0, len(dicoms)):
+#         dicom_ = dicoms[index]
+#         if stack_position_tag not in dicom_:
+#             stack_index = 0
+#         else:
+#             stack_index = dicom_[stack_position_tag].value - 1
+#         while len(grouped_dicoms) <= stack_index:
+#             grouped_dicoms.append([])
+#         grouped_dicoms[stack_index].append(dicom_)
+#
+#     return grouped_dicoms
+
 
 def _classic_get_full_block(grouped_dicoms):
     """
